from hasty.inference.attributer import Attributer
from hasty.inference.detector import Detector
from hasty.inference.instance_segmentor import InstanceSegmentor
from hasty.inference.semantic_segmentor import SemanticSegmentor

__all__ = [
    'Attributer',
    'Detector',
    'InstanceSegmentor',
    'SemanticSegmentor'
]
